/**
 * @packageDocumentation
 * React components, hooks and utils used to connect to {@link @af-utils/virtual-core#VirtualScroller}
 */

import type { ComponentProps } from 'react';
import { ElementType } from 'react';
import type { ReactNode } from 'react';
import { VirtualScroller } from '@af-utils/virtual-core';
import { VirtualScrollerEvent } from '@af-utils/virtual-core';
import { VirtualScrollerInitialParams } from '@af-utils/virtual-core';
import type { VirtualScrollerScrollElement } from '@af-utils/virtual-core';

/**
 * @public
 * React component.
 * Small abstraction, which in 90% cases allows to avoid hook boilerplate.
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
export declare function List<Component extends ElementType = "div">(props: ListProps<Component>): JSX.Element;

/**
 * @public
 * Props passed to List item
 */
export declare interface ListItemProps {
    model: VirtualScroller;
    /** item index */
    i: number;
    data?: any;
    offset?: number;
}

/**
 * @public
 * {@link List} component props
 */
export declare type ListProps<T extends ElementType = "div"> = Omit<ComponentProps<T>, "children" | "ref"> & {
    model: VirtualScroller;
    children: ElementType<ListItemProps>;
    itemData?: ListItemProps["data"];
    getKey?: (index: number, itemData: ListItemProps["data"]) => string | number;
    component?: T;
    header?: JSX.Element | null;
    footer?: JSX.Element | null;
};

/**
 * @public
 * Map current visible items
 * @param model - {@link @af-utils/virtual-core#VirtualScroller} instance
 * @param cb - callback
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
export declare function mapVisibleRange<T>(model: VirtualScroller, cb: (index: number) => T): T[];

/**
 * @public
 * Map current visible items and provide offset for each item.
 *
 * @remarks
 * Used in scenarios, when each item is absolutely positioned
 *
 * @param model - {@link @af-utils/virtual-core#VirtualScroller} instance
 * @param cb - callback
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
export declare function mapVisibleRangeWithOffset<T>(model: VirtualScroller, cb: (index: number, offset: number) => T): T[];

/**
 * @public
 * React component.
 * Rerenders only on certain {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)}.
 * Allows to optimize performance.
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
export declare function Subscription(props: SubscriptionProps): ReactNode;

/**
 * @public
 * {@link Subscription} component props
 */
export declare interface SubscriptionProps {
    model: VirtualScroller;
    children: () => ReactNode;
    events: readonly VirtualScrollerEvent[] | VirtualScrollerEvent[];
}

/**
 * @public
 * React hook.
 * Rerenders component when one of {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} gets emitted.
 * Usually {@link Subscription} is a better alternative
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
export declare function useComponentSubscription(model: VirtualScroller, events: readonly VirtualScrollerEvent[] | VirtualScrollerEvent[]): void;

/**
 * @public
 * React hook.
 * Synchronizes scroller with model.
 *
 * @remarks
 * Should be used in window-scroll cases, otherwise `ref={el => model.setScroller( el )}` is preferrable.
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
export declare function useScroller(model: VirtualScroller, scroller: VirtualScrollerScrollElement | null): void;

/**
 * @public
 * React hook.
 * Allows to subscribe to {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} without unnecessary rerenders.
 *
 * @remarks
 * For example can be used in load-on-demand.
 *
 * @param model - {@link @af-utils/virtual-core#VirtualScroller} instance
 * @param events - array of {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} to subscribe
 * @param callBack - callback to be called
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
export declare function useSubscription(model: VirtualScroller, events: readonly VirtualScrollerEvent[] | VirtualScrollerEvent[], callBack: () => void): void;

/**
 * @public
 * React hook.
 * Optimal CSS markup for virtual scroll is not intuitive.
 * Use this hook to avoid unneeded boilerplate.
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
export declare function useSyncedStyles(model: VirtualScroller): [
(outerRef: HTMLElement | null) => void,
(innerRef: HTMLElement | null) => void
];

/**
 * @public
 * React hook.
 * Calls {@link useVirtualModel} and synchronizes it with props
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
export declare function useVirtual(params: VirtualScrollerInitialParams): VirtualScroller;

/**
 * @public
 * React hook.
 * Creates and stores one {@link @af-utils/virtual-core#VirtualScroller} instance.
 * It does not get recreated during component lifecycle.
 * Normally {@link useVirtual} should be used.
 *
 * @privateRemarks
 * TODO: convert to arrow function when https://github.com/microsoft/rushstack/issues/1629 gets solved
 */
export declare function useVirtualModel(params: VirtualScrollerInitialParams): VirtualScroller;

export { }
